/********************************************************************************/
/*																				*/
/*	File name: fl_17705.c														*/
/*	  This is flash memory driver for S1C17705 flash memory						*/
/*																				*/
/*	Revision history															*/
/*		2014/11/17			start.											    */
/*                                                                              */
/*	Copyright (C) SEIKO EPSON CORPORATION 2014. All rights reserved.         	*/
/********************************************************************************/

/* Include */
#include	"fl_17703.h"

/* Prototype */
int	flash_erase(unsigned long, unsigned char, unsigned char);
int	flash_load(unsigned long, unsigned long,unsigned char*);
void voltage_up(void);
void voltage_down(void);
void wait(void);

/* extern */
extern int chip_erase(unsigned long, unsigned char);
extern int emb_program(unsigned long, unsigned short);
extern int	flsVerifyCheck(unsigned long, unsigned short);

/* Global */
unsigned long flsCtrlReg = 0xFFFFFFFF;			// Flash Control Register
extern unsigned long sSector[FLS_SCTMAX];		// Sector Address

/*************************************************************
	erasechk()

	description:
		Flash memory erase check.

	argument	:
		start     : Flash erase start block
		end       : Flash erase end block

	return		:
		FLS_SUCCESS  (0) : Erase Successful
		FLS_E_VERIFY (1) : Erase Verify Error
**************************************************************/
static int erasechk(unsigned long start_addr, unsigned long end_addr)
{
	int		ret = FLS_E_SUCCESS;

	for(;start_addr < end_addr; start_addr+=2)
	{
		if(*(volatile unsigned short *)(start_addr)!= 0xFFFF)
		{
			ret = FLS_E_VERIFY;
			break;
		}
	}

	return ret;
}

/*******************************************************************************
 * flash_erase
 *   Type     :	int
 *   Ret val  :	Flash memory sector erase result
 *                FLS_SUCCESS  (0) : Erase Successful
 *                FLS_E_VERIFY (1) : Erase Verify Error
 *                FLS_E_PARAM1 (3) : Start Sector Parameter Error
 *                FLS_E_PARAM2 (4) : End Sector Parameter Error
 *                FLS_E_REGERR (6) : Control Register Error
 *   Argument :	ulCtrlReg          ... Flash control register
 *             	lStart             ... Flash erase start block
 *             	lEnd               ... Flash erase end block
 *   Function :	Flash memory erase.
 *******************************************************************************/
int	flash_erase(unsigned long ulCtrlReg, unsigned char ucStart, unsigned char ucEnd)
{
	int	iRetVal;							// Return Value
	unsigned long	ulSectAddr;				// Sector Top Address

	flsCtrlReg = ulCtrlReg;					// Change control regester

	if(ulCtrlReg != FLS_TOPADDR)			// Check control register
	{
		return FLS_E_REGERR;				// Control register error !!
	}

	// Check Start Sector parameter
	if(ucStart > (FLS_SCTMAX - 1)){
		// Start Sector parameter error !!
		return FLS_E_PARAM1;
	}

	// Check End Sector parameter
	if(ucEnd > (FLS_SCTMAX - 1)){
		// End Sector paramemeter error !!
		return FLS_E_PARAM2;
	}

	// Check End Sector parameter
	if(ucStart > ucEnd){
		// End Sector paramemeter error !!
		return FLS_E_PARAM2;
	}

	iRetVal = FLS_E_SUCCESS;

	voltage_up();

	if(ucStart == 0)														// Chip erase
	{
		iRetVal = chip_erase(ulCtrlReg, CHIP_ERASE);						// Flash memory erase command sequence
	}
	else																	// Blocks or sectors erase
	{
		for(ucStart--, ucEnd--; (iRetVal == FLS_E_SUCCESS)&&(ucStart <= ucEnd); ucStart++)
		{
			ulSectAddr = sSector[ucStart] + ulCtrlReg;						// Calculate block top address
			iRetVal = chip_erase(ulSectAddr, SECT_ERASE);					// Flash memory erase command sequence
			iRetVal = erasechk(ulSectAddr, ulSectAddr + FLS_SCTSIZE);
		}
	}

	voltage_down();

	return(iRetVal);
}

/*******************************************************************************
 * flash_load
 *   Ret val  :	Flash memory byte result
 *                FLS_SUCCESS  (0) : Write Successful
 *                FLS_E_VERIFY (1) : Write Verify Error
 *   Argument :	unsigned long ulProgAddr ... Program address
 *             	unsigned long ulSize     ... Write data size
 *             	unsigned char* 	 		 ... Write data
 *   Function :	load to Flash memory.
 *******************************************************************************/
int	flash_load(unsigned long ulProgAddr, unsigned long ulSize, unsigned char* pData)
{
	int	iRetVal;
	unsigned long	ulWrtSize = 0;
	unsigned short	usWrtData;
	unsigned short	usRdData;

	flsCtrlReg = FLS_TOPADDR;									// set top address of flash memory

	iRetVal = FLS_E_SUCCESS;

	voltage_up();												// change VD1 to flash erase/program mode

	while(ulSize > ulWrtSize)									// check write size
	{
		if((ulProgAddr % 2) == 1)								// check odd address
		{
			ulProgAddr--;										// decrease address(odd -> even)

			usRdData = *(volatile unsigned short *)ulProgAddr;	// read destination address

			usWrtData = usRdData & (((unsigned short)pData[ulWrtSize] << 8) | FLS_FILL_L);	// create write data

			ulWrtSize++;										// increase write size
		}

		else if((ulSize - ulWrtSize) == 1)						// check the rest of byte
		{
			usRdData = *(volatile unsigned short *)ulProgAddr;	// read destination address

			usWrtData = usRdData & ((unsigned short)pData[ulWrtSize] | FLS_FILL_H);			// create write data

			ulWrtSize++;										// increase write size
		}

		else
		{
			usWrtData = (unsigned short)pData[ulWrtSize] | ((unsigned short)pData[ulWrtSize+1] << 8);		// create write data

			ulWrtSize += 2;										// increase write size
		}

		if(usWrtData != FLS_INIDAT)								// check write data = FFFFh
		{
			usRdData = *(volatile unsigned short *)ulProgAddr;	// read destination address

			if((usRdData & usWrtData) == usWrtData)				// check recordable address
			{
				iRetVal = emb_program(ulProgAddr, usWrtData);	// write to flash memory

				if(iRetVal != FLS_E_SUCCESS)
				{
					break;
				}
			}
			else
			{
				iRetVal = FLS_E_VERIFY;
				break;
			}
		}
		ulProgAddr += 2;										// increase destination address
	}

	voltage_down();												// change VD1 to normal mode

	return iRetVal;
}

/*******************************************************************************
 * voltage_up
 *   Type     :	void
 *   Ret val  :	none
 *   Argument :	void
 *   Function :	Change VD1 to flash erase/program mode function
 *******************************************************************************/
void voltage_up(void)
{
	unsigned char ClkSrc;
	ClkSrc = OSC_SRC;
	ClkSrc &= 0x03;
	ClkSrc <<= 2;

	MISC_DMODE2 = 0x01;			//Run select in debug mode

	SVD_IMSK = 0x00;
	OSC_SVD = 0x01 | ClkSrc;	//clock enable
	SVD_CMP = 0x0b;				//2.8V
	SVD_EN = 0x01;				//SVD enable

	VD1_CTL |= VD1_CTL_VD1MD;
	wait();						//IOSC2.7MHz(Max3.24MHz)/8ms wait

	if(SVD_RSLT)				//Low
	{
		MISC_PROT = 0x96;
		VD1_SEL = 0x01;
	}

	SVD_EN = 0x00;				//SVD disable
	OSC_SVD = 0x04;
	SVD_CMP = 0;

	MISC_DMODE2 = 0x00;			//Stop select in debug mode
}

/*******************************************************************************
 * voltage_down
 *   Type     :	void
 *   Ret val  :	none
 *   Argument :	void
 *   Function :	change VD1 to normal mode function
 *******************************************************************************/
void voltage_down(void)
{
	VD1_SEL = 0x00;
	VD1_CTL &= ~VD1_CTL_VD1MD;
	MISC_PROT = 0x00;
	wait();
}

/*******************************************************************************
 * wait
 *   Type     :	void
 *   Ret val  :	none
 *   Argument :	void
 *   Function :	Stabilization wait for changing VD1 function
 *
 *	 history : 2007/11/01    start
 *
 *******************************************************************************/
void wait(void)
{
	unsigned long cnt;

	for(cnt=0;cnt < SOFT_WAIT; cnt++)
	{
		/* wait */
	}

}
