/********************************************************************************
 *
 *  fl_17703exe.c
 *
 * 	This is 16 bit device flash memory controller for S1C17703
 *
 *	history : 2010/04/13    start
 *
 *  Copyright (C) SEIKO EPSON CORPORATION 2010-2014. All rights reserved.
********************************************************************************/

/* Include */
#include	"fl_17703.h"

/* Prototype */
int chip_erase(unsigned long, unsigned char);
int emb_program(unsigned long, unsigned short);
int	flsVerifyCheck(unsigned long, unsigned short);
unsigned short dq_poll(unsigned long);

/* Extern */
extern unsigned long flsCtrlReg;

/*******************************************************************************
 * chip_erase
 *   Type     :	void
 *   Ret val  :	Verify check result code
 *   Argument :	unsigned long ulAddr ... Erase sector address
 *                                       Chip erase mode is not use.
 *              unsigned long ulMode ... Erase mode flag
 *                                       1 : Chip erase
 *                                       2 : Sector erase
 *   Function :	Flash memory sector erase command sequence.
 *******************************************************************************/
int chip_erase(unsigned long ulAddr, unsigned char ulMode)
{
	int	iRetVal;

	// Erase command sequence
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM1_ADDR)) = FLASH_ERASE1;
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM2_ADDR)) = FLASH_ERASE2;
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM1_ADDR)) = FLASH_ERASE3;
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM1_ADDR)) = FLASH_ERASE4;
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM2_ADDR)) = FLASH_ERASE5;

	switch(ulMode)
	{
		case	1:						// Set chip erase command
			*((volatile unsigned short *)(flsCtrlReg + FLASH_COM1_ADDR)) = FLASH_ERASE6;
			break;
		case	2:						// Set sector erase command
			*((volatile unsigned short *)ulAddr) = FLASH_ERASE6_S;
			break;
		default:
			break;
	}

	iRetVal = flsVerifyCheck(ulAddr, FLS_INIDAT);

	return 	iRetVal;
}

/*******************************************************************************
 * emb_program
 *   Type     :	void
 *   Ret val  :	Verify check result code
 *   Argument :	unsigned long ulProgAddr ... Program address
 *              unsigned long ulWrtData  ... Word program data
 *   Function :	Flash memory embedded algorithm byte program sequence.
 *******************************************************************************/
int	emb_program(unsigned long ulProgAddr, unsigned short usWrtData)
{
	int	iRetVal;

	// Program command sequence
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM1_ADDR)) = FLASH_PROG1;
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM2_ADDR)) = FLASH_PROG2;
	*((volatile unsigned short *)(flsCtrlReg + FLASH_COM1_ADDR)) = FLASH_PROG3;
	*((volatile unsigned short *)ulProgAddr) = usWrtData;

	iRetVal = flsVerifyCheck(ulProgAddr, usWrtData);

	return 	iRetVal;
}

/*******************************************************************************
 * flsVerifyCheck
 *   Type     :	int
 *   Ret val  :	Verify check result code
 *                FLS_SUCCESS  (0) : Erase/Write Successful
 *                FLS_E_VERIFY (1) : Erase/Write Verify Error
 *   Argument :	unsigned long ulChkAddr ... Verify check address
 *              unsigned long ulChkData ... Verify check data
 *   Function :	Erase/Write verify check function
 *******************************************************************************/
int	flsVerifyCheck(unsigned long ulChkAddr, unsigned short ulChkData)
{
	int	iRetVal;						// Return Value
	unsigned short	ulOldTgl;			// Old Toggle Bit
	unsigned short	ulNewTgl;			// New Toggle Bit
	unsigned short	ulPolling;			// Polling Data

	iRetVal = FLS_E_SUCCESS;

	////////////////////////////
	// TOGGLE BIT CHECK : DQ6 //
	////////////////////////////
	ulOldTgl = FLS_INIDAT;
	while(1)
	{
		ulNewTgl = dq_poll(ulChkAddr) & FLS_B_DQ6;		// Toggle bit check
		if(ulOldTgl != ulNewTgl)
		{
			ulOldTgl = ulNewTgl;
		}
		else{
			break;
		}
	}

    ///////////////////////
	// DATA VERIFY CHECK //
	///////////////////////
	ulPolling  = dq_poll(ulChkAddr);					// Data polling
	if(ulPolling != ulChkData)							// Verify check
	{
		iRetVal = FLS_E_VERIFY;							// >>> Write verify error !!
	}

	return(iRetVal);
}

/*******************************************************************************
 * dq_poll
 *   Type     :	unsigned short
 *   Ret val  :	Data poll status
 *   Argument :	unsigned long ulAddr ... Flash memory address
 *   Function :	Flash memory reset.
 *******************************************************************************/
unsigned short	dq_poll(unsigned long ulAddr)
{
	unsigned long	ulPolling;							// Polling

	ulPolling = *((volatile unsigned short *)ulAddr);	// Status or data polling.

	return((unsigned short)ulPolling);
}
